﻿// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#if !defined(_IW_GAME_H_)
#define _IW_GAME_H_

// Version 0.320
#define	IW_GAME_VERSION		0x00340

#include "IwGameUtil.h"
#include "IwGameActor.h"
#include "IwGameScene.h"
#include "IwGameCamera.h"
#include "IwGameInput.h"
#include "IwGameAudio.h"
#include "IwGameXoml.h"
#include "IwGameResources.h"

//
//
//
//
// CIwGame is a acts as a controller for the whole game. You should derive your own version of this class
// 
//
// 
//
class CIwGame : public IIwGameXomlResource
{
public:
	// Public access for scene iteration
	typedef CIwList<CIwGameScene*>::iterator	_Iterator;
	_Iterator					begin() { return Scenes.begin(); }
	_Iterator					end() { return Scenes.end(); }

protected:
	//// Properties
	CIwGameScene*				CurrentScene;			// Scene that has current input focus
	CIwGameScene*				NextScene;				// Scene that we wish to switch focus to
	CIwList<CIwGameScene*>		Scenes;					// A collection of game scenes
	CIwGameActor*				TouchFocus;				// Actor that has the current touch focus
	CIwColour					BackgroundColour;		// Background clear colour

public:
	void						addScene(CIwGameScene *scene, bool bring_to_front = true);
	void						removeScene(CIwGameScene* scene);
	void						removeScene(unsigned int name_hash);
	void						removeAllScenes(CIwGameScene* exclude_scene);
	void						removeAllScenes(unsigned int exclude_name_hash);
	CIwGameScene*				findScene(unsigned int name_hash);
	CIwGameScene*				findScene(const char* name);
	CIwGameScene*				findScene(int type);
	CIwGameScene*				getScene(int index);
	void						clearScenes();
	void						changeScene(CIwGameScene *new_scene);
	bool						changeScene(unsigned int name_hash);
	CIwGameScene*				getCurrentScene()						{ return CurrentScene; }
	void						setTouchFocus(CIwGameActor* focus)		{ TouchFocus = focus; }
	CIwGameActor*				getTouchFocus()							{ return TouchFocus; }
	//// Properties end

protected:
	CIwList<CIwGameScene*>		SceneRemovals;		// Scenes that were removed during this game frame
	uint64						LastFrameTime;		// The time at which the last frame ended
	bool						PrevTouch;
	void						ProcessEvents(CIwGameScene *scane);

public:
	virtual void				Init(bool enable_http = false, bool enable_extensions = true);
	virtual void				Release();
	virtual bool				Update();
	virtual void				Draw();
	virtual void				PostDraw();			// Called by Draw() after all rendering is finished, but before the frame bugger is swapped
	virtual void				Save() {}
	virtual void				Load() {}

private:
public:
	void						SetBackgroundColour(uint8 r, uint8 g, uint8 b, uint8 a);
	void						BringSceneToFront(CIwGameScene* scene);
	void						DisableFocus();										// Disables focus for all scenes
	void						SetAllTimelines(CIwGameAnimTimeline* timeline);		// Sets the timelines of all active scenes

	// Implementation of IIwGameXomlClass interface
	bool						LoadFromXoml(IIwGameXomlResource* parebt, bool load_children, CIwGameXmlNode* node);
};

class CIwGameCreator : public IIwGameXomlClassCreator
{
public:
	CIwGameCreator()
	{
		setClassName("Game");
	}
	IIwGameXomlResource* CreateInstance(IIwGameXomlResource* parent);
};


//
//
//
//
//  CIwGameXomlAction_Global - A collection of global actions
//
//
//
//
class CIwGameXomlAction_Global : public IIwGameXomlAction
{
public:
	enum ActionType
	{
		Action_PlayTimeline		= 0, 
		Action_StopTimeline		= 1, 
		Action_SetTimeline		= 2, 
		Action_PlaySound		= 3, 
		Action_PlayMusic		= 4, 
		Action_StopMusic		= 5, 
		Action_LoadXOML			= 6, 
		Action_SetVariable		= 7, 
		Action_AddVariable		= 8, 
		Action_Launch			= 9, 
		Action_SetBGColour		= 10, 
		Action_SetCurrentScene	= 11, 
		Action_BringtSceneToFront= 12, 
		Action_EnterValue		= 13, 
		Action_UpdateText		= 14, 
		Action_Exit				= 15, 
		Action_RemoveResource	= 16, 
		Action_RemoveResources	= 17, 
		Action_CallActions		= 18, 
		Action_Max				= 19, 
	};
protected:
	ActionType	Type;
	CIwGameXomlAction_Global() {}
public:
	CIwGameXomlAction_Global(ActionType type)
	{
		Type = type;
		switch (Type)
		{
		case Action_PlayTimeline:
			setActionName("playtimeline");
			break;
		case Action_StopTimeline:
			setActionName("stoptimeline");
			break;
		case Action_SetTimeline:
			setActionName("settimeline");
			break;
		case Action_PlaySound:
			setActionName("playsound");
			break;
		case Action_PlayMusic:
			setActionName("playmusic");
			break;
		case Action_StopMusic:
			setActionName("stopmusic");
			break;
		case Action_LoadXOML:
			setActionName("loadxoml");
			break;
		case Action_SetVariable:
			setActionName("setvar");
			break;
		case Action_AddVariable:
			setActionName("addvar");
			break;
		case Action_Launch:
			setActionName("launch");
			break;
		case Action_SetBGColour:
			setActionName("setbgcolour");
			break;
		case Action_SetCurrentScene:
			setActionName("setcurrentscene");
			break;
		case Action_BringtSceneToFront:
			setActionName("bringscenetofront");
			break;
		case Action_EnterValue:
			setActionName("entervalue");
			break;
		case Action_UpdateText:
			setActionName("updatetext");
			break;
		case Action_Exit:
			setActionName("exit");
			break;
		case Action_RemoveResource:
			setActionName("removeresource");
			break;
		case Action_RemoveResources:
			setActionName("removeresources");
			break;
		}
	}
	void Execute(IIwGameXomlResource* source, CIwGameAction* action);
};


#endif // _IW_GAME_H_
