// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#if !defined(_IW_GAME_RENDER_H_)
#define _IW_GAME_RENDER_H_


#include "IwGx.h"
#include "IwGxFont.h"

#include "IwGameUtil.h"
#include "IwGameImage.h"

class CIwGameFont;

struct CIwGameRender2dMaterial
{
	CIwGameImage*	Image;
};

struct CIwGameRender2dPrim
{
	CIwVec2*		Verts;
	CIwSVec2*		UVs;
	int				Material;
	CIwColour*		Colours;
};

//
//
//
// CIwGameRedner2d - Class that performs 2D rendering
//
// 
//
class CIwGameRender2d
{
public:
	CDEFINE_SINGLETON(CIwGameRender2d)

protected:
	// Properties
	CIwGameRender2dPrim*		Primitives;
	int							NextPrimitive;
	int							MaxPrimitives;
	CIwGameRender2dMaterial*	Materials;
	int*						MaterialUsedCounts;
	int							NextMaterial;
	int							MaxMaterials;
	int*						MaterialIndices;
	CIwSVec2**					BatchVerts;
	CIwSVec2**					BatchUVs;
	CIwColour**					BatchColours;
	uint16**					BatchIndices;
public:
	// Properties end

protected:
	static uint16	Indices[4];
	int				AddMaterial(CIwGameImage* image);
	
public:
	void			Init(int max_primitives = 1024, int max_materials = 100);
	void			Release();

	void			DrawImage(CIwVec2* verts, CIwSVec2* uvs, CIwGameImage* image, CIwColour colour, bool filter = true);
	void			DrawImage(CIwVec2* verts, CIwSVec2* uvs, CIwGameImage* image, CIwColour* colours, bool filter = true);

	void			DrawText(CIwGxFontPreparedData* prepared_text, CIwGameFont* font, CIwMat2D* transform, CIwColour& colour, bool filter = true);

	void			AddPrimtive(CIwVec2* verts, CIwSVec2* uvs, CIwGameImage* image, CIwColour* colours);
	void			BatchDrawPrims(bool filter = true);

};

#define IW_GAME_RENDER2D		CIwGameRender2d::getInstance()

#endif	// _IW_GAME_RENDER_H_
