// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#include "IwGameFont.h"
#include "IwGameResources.h"
#include "IwGameScene.h"

//
//
//
//
// CIwGameFont implementation
//
//
//
//

bool CIwGameFont::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	// Process image attributes
	CIwGameString name;
	CIwGameString location;

	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Name_Hash)
		{
			name = (*it)->GetValue();
			setName(name.c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::Tag_Hash)
			setTag((*it)->GetValue().c_str());
		else
		if (name_hash == CIwGameXomlNames::Location_Hash)
		{
			location = (*it)->GetValue().c_str();
		}
	}

	// Find resource group
	CIwGameResourceGroup* group = NULL;
	if (parent != NULL && parent->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
		group = (CIwGameResourceGroup*)((CIwGameScene*)parent)->getResourceManager()->findResource(location.c_str(), CIwGameXomlNames::ResourceGroup_Hash);
	else
		group = (CIwGameResourceGroup*)IW_GAME_GLOBAL_RESOURCES->getResourceManager()->findResource(location.c_str(), CIwGameXomlNames::ResourceGroup_Hash);

	if (group != NULL)
		Init(name.c_str(), group->getResourceGroup());
	else
	{
		CIwGameError::LogError("Error: XOML - Invalid font resource group name - ", location.c_str());
		return false;
	}

	// If we are declared inside a scene then font is local to the scene
	if (parent != NULL && parent->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
	{
		((CIwGameScene*)parent)->getResourceManager()->addResource(this);
	}
	else
	{
		IW_GAME_GLOBAL_RESOURCES->getResourceManager()->addResource(this);
	}

	return true;
}


