// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#include "IwGameXoml.h"
#include "IwGameUtil.h"
#include "IwGameString.h"
#include "IwGame.h"
#include "IwGameScene.h"
#include "IwGameActor.h"
#include "IwGameActorExt.h"
#include "IwGameActorParticles.h"
#include "IwGameAnim.h"
#include "IwGameShapes.h"
#include "IwGameBox2d.h"
#include "IwGameCamera.h"
#include "IwGameStyles.h"
#include "IwGameTemplates.h"
#include "IwGameActions.h"
#include "IwGameFont.h"
#include "IwGameModifier.h"
#include "IwGameModifierExt.h"


unsigned int CIwGameXomlNames::Game_Hash					= CIwGameString::CalculateHash("game");
unsigned int CIwGameXomlNames::Scene_Hash					= CIwGameString::CalculateHash("scene");
unsigned int CIwGameXomlNames::Actor_Hash					= CIwGameString::CalculateHash("actor");
unsigned int CIwGameXomlNames::ActorImage_Hash				= CIwGameString::CalculateHash("actorimage");
unsigned int CIwGameXomlNames::ActorText_Hash				= CIwGameString::CalculateHash("actortext");
unsigned int CIwGameXomlNames::ResourceGroup_Hash			= CIwGameString::CalculateHash("resourcegroup");
unsigned int CIwGameXomlNames::Animation_Hash				= CIwGameString::CalculateHash("animation");
unsigned int CIwGameXomlNames::Box2dMaterial_Hash			= CIwGameString::CalculateHash("box2dmaterial");
unsigned int CIwGameXomlNames::Joints_Hash					= CIwGameString::CalculateHash("joints");
unsigned int CIwGameXomlNames::Joint_Hash					= CIwGameString::CalculateHash("joint");
unsigned int CIwGameXomlNames::Shape_Hash					= CIwGameString::CalculateHash("shape");
unsigned int CIwGameXomlNames::Actions_Hash					= CIwGameString::CalculateHash("actions");
unsigned int CIwGameXomlNames::Camera_Hash					= CIwGameString::CalculateHash("camera");
unsigned int CIwGameXomlNames::Image_Hash					= CIwGameString::CalculateHash("image");
unsigned int CIwGameXomlNames::Brush_Hash					= CIwGameString::CalculateHash("brush");
unsigned int CIwGameXomlNames::Font_Hash					= CIwGameString::CalculateHash("font");
unsigned int CIwGameXomlNames::Timeline_Hash				= CIwGameString::CalculateHash("timeline");
unsigned int CIwGameXomlNames::Style_Hash					= CIwGameString::CalculateHash("style");
unsigned int CIwGameXomlNames::Variable_Hash				= CIwGameString::CalculateHash("variable");
unsigned int CIwGameXomlNames::Bindings_Hash				= CIwGameString::CalculateHash("bindings");
unsigned int CIwGameXomlNames::Binding_Hash					= CIwGameString::CalculateHash("binding");
unsigned int CIwGameXomlNames::Modifier_Hash				= CIwGameString::CalculateHash("modifier");

unsigned int CIwGameXomlNames::Count_Hash					= CIwGameString::CalculateHash("count");
unsigned int CIwGameXomlNames::Action_Hash					= CIwGameString::CalculateHash("action");
unsigned int CIwGameXomlNames::Name_Hash					= CIwGameString::CalculateHash("name");
unsigned int CIwGameXomlNames::File_Hash					= CIwGameString::CalculateHash("file");
unsigned int CIwGameXomlNames::Current_Hash					= CIwGameString::CalculateHash("current");
unsigned int CIwGameXomlNames::CanvasSize_Hash				= CIwGameString::CalculateHash("canvassize");
unsigned int CIwGameXomlNames::FixAspect_Hash				= CIwGameString::CalculateHash("fixaspect");
unsigned int CIwGameXomlNames::LockWidth_Hash				= CIwGameString::CalculateHash("lockwidth");
unsigned int CIwGameXomlNames::Extents_Hash					= CIwGameString::CalculateHash("extents");
unsigned int CIwGameXomlNames::AllowSuspend_Hash			= CIwGameString::CalculateHash("allowsuspend");
unsigned int CIwGameXomlNames::Clipping_Hash				= CIwGameString::CalculateHash("clipping");
unsigned int CIwGameXomlNames::ClipStatic_Hash				= CIwGameString::CalculateHash("clipstatic");
unsigned int CIwGameXomlNames::Active_Hash					= CIwGameString::CalculateHash("active");
unsigned int CIwGameXomlNames::Visible_Hash					= CIwGameString::CalculateHash("visible");
unsigned int CIwGameXomlNames::Layers_Hash					= CIwGameString::CalculateHash("layers");
unsigned int CIwGameXomlNames::Colliders_Hash				= CIwGameString::CalculateHash("colliders");
unsigned int CIwGameXomlNames::Type_Hash					= CIwGameString::CalculateHash("type");
unsigned int CIwGameXomlNames::Position_Hash				= CIwGameString::CalculateHash("position");
unsigned int CIwGameXomlNames::Pitch_Hash					= CIwGameString::CalculateHash("pitch");
unsigned int CIwGameXomlNames::Origin_Hash					= CIwGameString::CalculateHash("origin");
unsigned int CIwGameXomlNames::Velocity_Hash				= CIwGameString::CalculateHash("velocity");
unsigned int CIwGameXomlNames::VelocityDamping_Hash			= CIwGameString::CalculateHash("velocitydamping");
unsigned int CIwGameXomlNames::Angle_Hash					= CIwGameString::CalculateHash("angle");
unsigned int CIwGameXomlNames::AngularVelocity_Hash			= CIwGameString::CalculateHash("angularvelocity");
unsigned int CIwGameXomlNames::AngularVelocityDamping_Hash	= CIwGameString::CalculateHash("angularvelocitydamping");
unsigned int CIwGameXomlNames::Depth_Hash					= CIwGameString::CalculateHash("depth");
unsigned int CIwGameXomlNames::DepthVelocity_Hash			= CIwGameString::CalculateHash("depthvelocity");
unsigned int CIwGameXomlNames::DepthVelocityDamping_Hash	= CIwGameString::CalculateHash("depthvelocitydamping");
unsigned int CIwGameXomlNames::Scale_Hash					= CIwGameString::CalculateHash("scale");
unsigned int CIwGameXomlNames::ScaleX_Hash					= CIwGameString::CalculateHash("scalex");
unsigned int CIwGameXomlNames::ScaleY_Hash					= CIwGameString::CalculateHash("scaley");
unsigned int CIwGameXomlNames::ScaleVelocity_Hash			= CIwGameString::CalculateHash("scalevelocity");
unsigned int CIwGameXomlNames::ScaleVelocityDamping_Hash	= CIwGameString::CalculateHash("scalevelocitydamping");
unsigned int CIwGameXomlNames::Colour_Hash					= CIwGameString::CalculateHash("colour");
unsigned int CIwGameXomlNames::ColourVelocity_Hash			= CIwGameString::CalculateHash("colourvelocity");
unsigned int CIwGameXomlNames::ColourVelocityDamping_Hash	= CIwGameString::CalculateHash("colourvelocitydamping");
unsigned int CIwGameXomlNames::Color_Hash					= CIwGameString::CalculateHash("color");
unsigned int CIwGameXomlNames::Layer_Hash					= CIwGameString::CalculateHash("layer");
unsigned int CIwGameXomlNames::Draggable_Hash				= CIwGameString::CalculateHash("draggable");
unsigned int CIwGameXomlNames::Collidable_Hash				= CIwGameString::CalculateHash("collidable");
unsigned int CIwGameXomlNames::CollisionSize_Hash			= CIwGameString::CalculateHash("collisionsize");
unsigned int CIwGameXomlNames::CollisionRect_Hash			= CIwGameString::CalculateHash("collisionrect");
unsigned int CIwGameXomlNames::WrapPosition_Hash			= CIwGameString::CalculateHash("wrapposition");
unsigned int CIwGameXomlNames::Size_Hash					= CIwGameString::CalculateHash("size");
unsigned int CIwGameXomlNames::Text_Hash					= CIwGameString::CalculateHash("text");
unsigned int CIwGameXomlNames::GroupFile_Hash				= CIwGameString::CalculateHash("groupfile");
unsigned int CIwGameXomlNames::Frame_Hash					= CIwGameString::CalculateHash("frame");
unsigned int CIwGameXomlNames::Atlas_Hash					= CIwGameString::CalculateHash("atlas");
unsigned int CIwGameXomlNames::Value_Hash					= CIwGameString::CalculateHash("value");
unsigned int CIwGameXomlNames::Time_Hash					= CIwGameString::CalculateHash("time");
unsigned int CIwGameXomlNames::Repeat_Hash					= CIwGameString::CalculateHash("repeat");
unsigned int CIwGameXomlNames::Duration_Hash				= CIwGameString::CalculateHash("duration");
unsigned int CIwGameXomlNames::SpawnDelay_Hash				= CIwGameString::CalculateHash("spawndelay");
unsigned int CIwGameXomlNames::SrcRect_Hash					= CIwGameString::CalculateHash("srcrect");
unsigned int CIwGameXomlNames::Location_Hash				= CIwGameString::CalculateHash("location");
unsigned int CIwGameXomlNames::Resource_Hash				= CIwGameString::CalculateHash("resource");
unsigned int CIwGameXomlNames::Tag_Hash						= CIwGameString::CalculateHash("tag");
unsigned int CIwGameXomlNames::Preload_Hash					= CIwGameString::CalculateHash("preload");
unsigned int CIwGameXomlNames::Blocking_Hash				= CIwGameString::CalculateHash("blocking");
unsigned int CIwGameXomlNames::AutoPlay_Hash				= CIwGameString::CalculateHash("autoplay");
unsigned int CIwGameXomlNames::Anim_Hash					= CIwGameString::CalculateHash("anim");
unsigned int CIwGameXomlNames::StartAtTime_Hash				= CIwGameString::CalculateHash("startattime");
unsigned int CIwGameXomlNames::Delta_Hash					= CIwGameString::CalculateHash("delta");
unsigned int CIwGameXomlNames::Interpolate_Hash				= CIwGameString::CalculateHash("interpolate");
unsigned int CIwGameXomlNames::Easing_Hash					= CIwGameString::CalculateHash("easing");
unsigned int CIwGameXomlNames::Target_Hash					= CIwGameString::CalculateHash("target");
unsigned int CIwGameXomlNames::TargetA_Hash					= CIwGameString::CalculateHash("targeta");
unsigned int CIwGameXomlNames::TargetB_Hash					= CIwGameString::CalculateHash("targetb");
unsigned int CIwGameXomlNames::Width_Hash					= CIwGameString::CalculateHash("width");
unsigned int CIwGameXomlNames::Height_Hash					= CIwGameString::CalculateHash("height");
unsigned int CIwGameXomlNames::Radius_Hash					= CIwGameString::CalculateHash("radius");
unsigned int CIwGameXomlNames::Points_Hash					= CIwGameString::CalculateHash("points");
unsigned int CIwGameXomlNames::Point_Hash					= CIwGameString::CalculateHash("point");
unsigned int CIwGameXomlNames::Particle_Hash				= CIwGameString::CalculateHash("particle");
unsigned int CIwGameXomlNames::Data_Hash					= CIwGameString::CalculateHash("data");
unsigned int CIwGameXomlNames::Density_Hash					= CIwGameString::CalculateHash("density");
unsigned int CIwGameXomlNames::Friction_Hash				= CIwGameString::CalculateHash("friction");
unsigned int CIwGameXomlNames::Restitution_Hash				= CIwGameString::CalculateHash("restitution");
unsigned int CIwGameXomlNames::IsBullet_Hash				= CIwGameString::CalculateHash("isbullet");
unsigned int CIwGameXomlNames::FixedRotation_Hash			= CIwGameString::CalculateHash("fixedrotation");
unsigned int CIwGameXomlNames::GravityScale_Hash			= CIwGameString::CalculateHash("gravityscale");
unsigned int CIwGameXomlNames::COM_Hash						= CIwGameString::CalculateHash("com");
unsigned int CIwGameXomlNames::Gravity_Hash					= CIwGameString::CalculateHash("gravity");
unsigned int CIwGameXomlNames::DoSleep_Hash					= CIwGameString::CalculateHash("dosleep");
unsigned int CIwGameXomlNames::Mode_Hash					= CIwGameString::CalculateHash("mode");
unsigned int CIwGameXomlNames::WorldScale_Hash				= CIwGameString::CalculateHash("worldscale");
unsigned int CIwGameXomlNames::Sensor_Hash					= CIwGameString::CalculateHash("sensor");
unsigned int CIwGameXomlNames::CollisionFlags_Hash			= CIwGameString::CalculateHash("collisionflags");
unsigned int CIwGameXomlNames::Set_Hash						= CIwGameString::CalculateHash("set");
unsigned int CIwGameXomlNames::Property_Hash				= CIwGameString::CalculateHash("property");
unsigned int CIwGameXomlNames::HitTest_Hash					= CIwGameString::CalculateHash("hittest");
unsigned int CIwGameXomlNames::FlipX_Hash					= CIwGameString::CalculateHash("flipx");
unsigned int CIwGameXomlNames::FlipY_Hash					= CIwGameString::CalculateHash("flipy");
unsigned int CIwGameXomlNames::LinkedTo_Hash				= CIwGameString::CalculateHash("linkedto");
unsigned int CIwGameXomlNames::Template_Hash				= CIwGameString::CalculateHash("template");
unsigned int CIwGameXomlNames::Prefix_Hash					= CIwGameString::CalculateHash("prefix");
unsigned int CIwGameXomlNames::AlignH_Hash					= CIwGameString::CalculateHash("alignh");
unsigned int CIwGameXomlNames::AlignV_Hash					= CIwGameString::CalculateHash("alignv");
unsigned int CIwGameXomlNames::Wrap_Hash					= CIwGameString::CalculateHash("wrap");
unsigned int CIwGameXomlNames::Italic_Hash					= CIwGameString::CalculateHash("italic");
unsigned int CIwGameXomlNames::Underline_Hash				= CIwGameString::CalculateHash("underline");
unsigned int CIwGameXomlNames::Method_Hash					= CIwGameString::CalculateHash("method");
unsigned int CIwGameXomlNames::Param1_Hash					= CIwGameString::CalculateHash("param1");
unsigned int CIwGameXomlNames::Param2_Hash					= CIwGameString::CalculateHash("param2");
unsigned int CIwGameXomlNames::Param3_Hash					= CIwGameString::CalculateHash("param3");
unsigned int CIwGameXomlNames::Param4_Hash					= CIwGameString::CalculateHash("param4");
unsigned int CIwGameXomlNames::Condition_Hash				= CIwGameString::CalculateHash("condition");
unsigned int CIwGameXomlNames::Batch_Hash					= CIwGameString::CalculateHash("batch");
unsigned int CIwGameXomlNames::AllowFocus_Hash				= CIwGameString::CalculateHash("allowfocus");
unsigned int CIwGameXomlNames::TouchPanX_Hash				= CIwGameString::CalculateHash("touchpanx");
unsigned int CIwGameXomlNames::TouchPanY_Hash				= CIwGameString::CalculateHash("touchpany");
unsigned int CIwGameXomlNames::ActorB_Hash					= CIwGameString::CalculateHash("actorb");
unsigned int CIwGameXomlNames::OffsetA_Hash					= CIwGameString::CalculateHash("offseta");
unsigned int CIwGameXomlNames::OffsetB_Hash					= CIwGameString::CalculateHash("offsetb");
unsigned int CIwGameXomlNames::GroundA_Hash					= CIwGameString::CalculateHash("grounda");
unsigned int CIwGameXomlNames::GroundB_Hash					= CIwGameString::CalculateHash("groundb");
unsigned int CIwGameXomlNames::SelfCollide_Hash				= CIwGameString::CalculateHash("selfcollide");
unsigned int CIwGameXomlNames::Distance_Hash				= CIwGameString::CalculateHash("distance");
unsigned int CIwGameXomlNames::Revolute_Hash				= CIwGameString::CalculateHash("revolute");
unsigned int CIwGameXomlNames::Prismatic_Hash				= CIwGameString::CalculateHash("prismatic");
unsigned int CIwGameXomlNames::Pulley_Hash					= CIwGameString::CalculateHash("pulley");
unsigned int CIwGameXomlNames::Gear_Hash					= CIwGameString::CalculateHash("gear");
unsigned int CIwGameXomlNames::Wheel_Hash					= CIwGameString::CalculateHash("wheel");
unsigned int CIwGameXomlNames::Rope_Hash					= CIwGameString::CalculateHash("rope");
unsigned int CIwGameXomlNames::Length_Hash					= CIwGameString::CalculateHash("length");
unsigned int CIwGameXomlNames::LengthA_Hash					= CIwGameString::CalculateHash("lengtha");
unsigned int CIwGameXomlNames::LengthB_Hash					= CIwGameString::CalculateHash("lengthb");
unsigned int CIwGameXomlNames::Ratio_Hash					= CIwGameString::CalculateHash("ratio");
unsigned int CIwGameXomlNames::Frequency_Hash				= CIwGameString::CalculateHash("frequency");
unsigned int CIwGameXomlNames::Damping_Hash					= CIwGameString::CalculateHash("damping");
unsigned int CIwGameXomlNames::MotorEnabled_Hash			= CIwGameString::CalculateHash("motorenabled");
unsigned int CIwGameXomlNames::LimitJoint_Hash				= CIwGameString::CalculateHash("limitjoint");
unsigned int CIwGameXomlNames::LowerLimit_Hash				= CIwGameString::CalculateHash("lowerlimit");
unsigned int CIwGameXomlNames::UpperLimit_Hash				= CIwGameString::CalculateHash("upperlimit");
unsigned int CIwGameXomlNames::MotorSpeed_Hash				= CIwGameString::CalculateHash("motorspeed");
unsigned int CIwGameXomlNames::MaxMotorTorque_Hash			= CIwGameString::CalculateHash("maxmotortorque");
unsigned int CIwGameXomlNames::MaxMotorForce_Hash			= CIwGameString::CalculateHash("maxmotorforce");
unsigned int CIwGameXomlNames::ReferenceAngle_Hash			= CIwGameString::CalculateHash("refangle");
unsigned int CIwGameXomlNames::Axis_Hash					= CIwGameString::CalculateHash("axis");
unsigned int CIwGameXomlNames::Force_Hash					= CIwGameString::CalculateHash("force");
unsigned int CIwGameXomlNames::LinearImpulse_Hash			= CIwGameString::CalculateHash("linearimpulse");
unsigned int CIwGameXomlNames::Torque_Hash					= CIwGameString::CalculateHash("torque");
unsigned int CIwGameXomlNames::AngularImpulse_Hash			= CIwGameString::CalculateHash("angularimpulse");
unsigned int CIwGameXomlNames::Solid_Hash					= CIwGameString::CalculateHash("solid");
unsigned int CIwGameXomlNames::Gradient_Hash				= CIwGameString::CalculateHash("gradient");
unsigned int CIwGameXomlNames::Activate_Hash				= CIwGameString::CalculateHash("activate");
unsigned int CIwGameXomlNames::Deactivate_Hash				= CIwGameString::CalculateHash("deactivate");
unsigned int CIwGameXomlNames::Toggle_Hash					= CIwGameString::CalculateHash("toggle");
unsigned int CIwGameXomlNames::Remove_Hash					= CIwGameString::CalculateHash("remove");

unsigned int CIwGameXomlNames::OnTapped_Hash				= CIwGameString::CalculateHash("ontapped");
unsigned int CIwGameXomlNames::OnBeginTouch_Hash			= CIwGameString::CalculateHash("onbegintouch");
unsigned int CIwGameXomlNames::OnEndTouch_Hash				= CIwGameString::CalculateHash("onendtouch");
unsigned int CIwGameXomlNames::OnSuspend_Hash				= CIwGameString::CalculateHash("onsuspend");
unsigned int CIwGameXomlNames::OnResume_Hash				= CIwGameString::CalculateHash("onresume");
unsigned int CIwGameXomlNames::OnLostFocus_Hash				= CIwGameString::CalculateHash("onlostfocus");
unsigned int CIwGameXomlNames::OnGainedFocus_Hash			= CIwGameString::CalculateHash("ongainedfocus");
unsigned int CIwGameXomlNames::OnStart_Hash					= CIwGameString::CalculateHash("onstart");
unsigned int CIwGameXomlNames::OnEnd_Hash					= CIwGameString::CalculateHash("onend");
unsigned int CIwGameXomlNames::OnRepeat_Hash				= CIwGameString::CalculateHash("onrepeat");
unsigned int CIwGameXomlNames::OnCreate_Hash				= CIwGameString::CalculateHash("oncreate");
unsigned int CIwGameXomlNames::OnDestroy_Hash				= CIwGameString::CalculateHash("ondestroy");
unsigned int CIwGameXomlNames::OnKeyBack_Hash				= CIwGameString::CalculateHash("onkeyback");
unsigned int CIwGameXomlNames::OnKeyMenu_Hash				= CIwGameString::CalculateHash("onkeymenu");

unsigned int CIwGameXomlNames::ChangeScene_Hash				= CIwGameString::CalculateHash("changescene");
unsigned int CIwGameXomlNames::SuspendScene_Hash			= CIwGameString::CalculateHash("suspendscene");
unsigned int CIwGameXomlNames::ResumeScene_Hash				= CIwGameString::CalculateHash("resumescene");
unsigned int CIwGameXomlNames::ShowScene_Hash				= CIwGameString::CalculateHash("showscene");
unsigned int CIwGameXomlNames::HideScene_Hash				= CIwGameString::CalculateHash("hidescene");
unsigned int CIwGameXomlNames::ActivateScene_Hash			= CIwGameString::CalculateHash("activatescene");
unsigned int CIwGameXomlNames::DeactivateScene_Hash			= CIwGameString::CalculateHash("deactivatescene");
unsigned int CIwGameXomlNames::KillScene_Hash				= CIwGameString::CalculateHash("killscene");
unsigned int CIwGameXomlNames::ShowActor_Hash				= CIwGameString::CalculateHash("showactor");
unsigned int CIwGameXomlNames::HideActor_Hash				= CIwGameString::CalculateHash("hideactor");
unsigned int CIwGameXomlNames::ActivateActor_Hash			= CIwGameString::CalculateHash("activateactor");
unsigned int CIwGameXomlNames::DeactivateActor_Hash			= CIwGameString::CalculateHash("deactivateactor");
unsigned int CIwGameXomlNames::KillActor_Hash				= CIwGameString::CalculateHash("killactor");
unsigned int CIwGameXomlNames::PlayTimeline_Hash			= CIwGameString::CalculateHash("playtimeline");
unsigned int CIwGameXomlNames::StopTimeline_Hash			= CIwGameString::CalculateHash("stoptimeline");
unsigned int CIwGameXomlNames::SetTimeline_Hash				= CIwGameString::CalculateHash("settimeline");
unsigned int CIwGameXomlNames::PlaySound_Hash				= CIwGameString::CalculateHash("playsound");
unsigned int CIwGameXomlNames::PlayMusic_Hash				= CIwGameString::CalculateHash("playmusic");
unsigned int CIwGameXomlNames::SetVar_Hash					= CIwGameString::CalculateHash("setvar");

unsigned int CIwGameXomlNames::bool_Hash					= CIwGameString::CalculateHash("bool");
unsigned int CIwGameXomlNames::float_Hash					= CIwGameString::CalculateHash("float");
unsigned int CIwGameXomlNames::int_Hash						= CIwGameString::CalculateHash("int");
unsigned int CIwGameXomlNames::vec2_Hash					= CIwGameString::CalculateHash("vec2");
unsigned int CIwGameXomlNames::vec3_Hash					= CIwGameString::CalculateHash("vec3");
unsigned int CIwGameXomlNames::vec4_Hash					= CIwGameString::CalculateHash("vec4");
unsigned int CIwGameXomlNames::rect_Hash					= CIwGameString::CalculateHash("rect");
unsigned int CIwGameXomlNames::string_Hash					= CIwGameString::CalculateHash("string");
unsigned int CIwGameXomlNames::condition_Hash				= CIwGameString::CalculateHash("condition");
unsigned int CIwGameXomlNames::box_Hash						= CIwGameString::CalculateHash("box");
unsigned int CIwGameXomlNames::circle_Hash					= CIwGameString::CalculateHash("circle");
unsigned int CIwGameXomlNames::polygon_Hash					= CIwGameString::CalculateHash("polygon");
unsigned int CIwGameXomlNames::static_Hash					= CIwGameString::CalculateHash("static");
unsigned int CIwGameXomlNames::dynamic_Hash					= CIwGameString::CalculateHash("dynamic");
unsigned int CIwGameXomlNames::kinematic_Hash				= CIwGameString::CalculateHash("kinematic");
unsigned int CIwGameXomlNames::Linear_Hash					= CIwGameString::CalculateHash("linear");
unsigned int CIwGameXomlNames::QuadIn_Hash					= CIwGameString::CalculateHash("quadin");
unsigned int CIwGameXomlNames::QuadOut_Hash					= CIwGameString::CalculateHash("quadout");
unsigned int CIwGameXomlNames::CubicIn_Hash					= CIwGameString::CalculateHash("cubicin");
unsigned int CIwGameXomlNames::CubicOut_Hash				= CIwGameString::CalculateHash("cubicout");
unsigned int CIwGameXomlNames::QuarticIn_Hash				= CIwGameString::CalculateHash("quarticin");
unsigned int CIwGameXomlNames::QuarticOut_Hash				= CIwGameString::CalculateHash("quarticout");
unsigned int CIwGameXomlNames::left_Hash					= CIwGameString::CalculateHash("left");
unsigned int CIwGameXomlNames::right_Hash					= CIwGameString::CalculateHash("right");
unsigned int CIwGameXomlNames::centre_Hash					= CIwGameString::CalculateHash("centre");
unsigned int CIwGameXomlNames::top_Hash						= CIwGameString::CalculateHash("top");
unsigned int CIwGameXomlNames::middle_Hash					= CIwGameString::CalculateHash("middle");
unsigned int CIwGameXomlNames::bottom_Hash					= CIwGameString::CalculateHash("bottom");


//
//
//
//
// CIwGameXomlResourceManager implementation
//
//
//
//
/*void CIwGameXomlResourceManager::addResource(const char* resource_name, const char* resource_filename, bool preload)
{
	IIwGameXomlResource* resource = new CIwGameResourceResource();
	resource->setResourceName(resource_name);
	resource->setResourceFilename(resource_filename);
	Resources.push_back(resource);

	if (preload)
	{
		resource->Load();
	}
}*/

void CIwGameXomlResourceManager::addResource(IIwGameXomlResource *resource)
{
#if defined(_DEBUG)
	if (findResource(resource->getNameHash(), resource->getClassTypeHash(), false))
	{
		CIwGameError::LogError("Warning: Resourcs already exists, duplicate was ignored - ", resource->getName().c_str());
		delete resource;
		return;
	}
#endif	//(_DEBUG)
	Resources.push_back(resource);
	resource->setParent(this);
	resource->setManaged(true);
}

void CIwGameXomlResourceManager::removeResource(IIwGameXomlResource* resource)
{
	removeResource(resource->getNameHash(), resource->getClassTypeHash());
}

void CIwGameXomlResourceManager::removeResource(unsigned int name_hash, unsigned int type_hash)
{
	// Renove named resource from the manager
	for (_Iterator it = Resources.begin(); it != Resources.end(); ++it)
	{
		if (name_hash == (*it)->getNameHash() && type_hash == (*it)->getClassTypeHash())
		{
			delete *it;
			Resources.erase(it);
			break;
		}
	}
}

void CIwGameXomlResourceManager::removeTaggedResources(unsigned int tag_hash)
{
	CIwList<IIwGameXomlResource*>		Removals;

	// Buildl ist of resources that are to be removed for tag
	for (_Iterator it = Resources.begin(); it != Resources.end(); ++it)
	{
		if (tag_hash == (*it)->getTagHash())
		{
			Removals.push_back(*it);
		}
	}

	// Remove the resources
	for (_Iterator it = Removals.begin(); it != Removals.end(); ++it)
	{
		for (_Iterator it2 = Resources.begin(); it2 != Resources.end(); ++it2)
		{
			if (*it2 == *it)
			{
				delete *it2;
				Resources.erase(it2);
				break;
			}
		}
	}
	Removals.clear();
}

void CIwGameXomlResourceManager::removeTaggedResources(const char* tag)
{
	removeTaggedResources(CIwGameString::CalculateHash(tag));
}

IIwGameXomlResource* CIwGameXomlResourceManager::findResource(unsigned int name_hash, unsigned int type_hash, bool global_search)
{
	// Find and return named image of specific type
	for (_Iterator it = Resources.begin(); it != Resources.end(); ++it)
	{
		if (name_hash == (*it)->getNameHash() && type_hash == (*it)->getClassTypeHash())
			return *it;
	}

	if (global_search)
	{
		// Not found so check global resources
		CIwGameXomlResourceManager* m = IW_GAME_GLOBAL_RESOURCES->getResourceManager();
		if (m != this)	// Prevent global resource manager from calling itself
			return m->findResource(name_hash, type_hash, false);
	}

	return NULL;
}

IIwGameXomlResource* CIwGameXomlResourceManager::findResource(const char* name, unsigned int type_hash, bool global_search)
{
	return findResource(CIwGameString::CalculateHash(name), type_hash, global_search);
}

IIwGameXomlResource* CIwGameXomlResourceManager::findResource(const char* name, const char* type, bool global_search)
{
	unsigned int name_hash = CIwGameString::CalculateHash(name);
	unsigned int type_hash = 0;

	if (type != NULL)
		type_hash = CIwGameString::CalculateHash(type);

	// Find and return named resource of specific type
	for (_Iterator it = Resources.begin(); it != Resources.end(); ++it)
	{
		if (type == NULL)
		{
			if (name_hash == (*it)->getNameHash())
				return *it;
		}
		else
		{
			if (name_hash == (*it)->getNameHash() && type_hash == (*it)->getClassTypeHash())
				return *it;
		}
	}

	if (global_search)
	{
		// Not found so check global resources
		CIwGameXomlResourceManager* m = IW_GAME_GLOBAL_RESOURCES->getResourceManager();
		if (m != this)	// Prevent global resource manager from calling itself
		{
			if (type != NULL)
				return m->findResource(name_hash, type_hash, false);
			else
				return m->findResource(name, type, false);
		}
	}

	return NULL;
}


void CIwGameXomlResourceManager::clearResources()
{
	// Remove and delete all images from the manager
	for (_Iterator it = Resources.begin(); it != Resources.end(); ++it)
	{
		delete *it;
	}
	Resources.clear();
}


//
// 
// 
//
// CIwGameXoml Implementation
// 
// 
// 
//
CDECLARE_SINGLETON(CIwGameXoml)

IIwGameXomlClassCreator* CIwGameXoml::findClass(unsigned int name_hash)
{
	for (_Iterator it = ClassCreators.begin(); it != ClassCreators.end(); ++it)
	{
		if ((*it)->getClassNameHash() == name_hash)
			return *it;
	}

	return NULL;
}

IIwGameXomlClassCreator* CIwGameXoml::findClass(const char* name)
{
	return findClass(CIwGameString::CalculateHash(name));
}

IIwGameXomlAction* CIwGameXoml::findAction(unsigned int name_hash)
{
	for (_ActionsIterator it = Actions.begin(); it != Actions.end(); ++it)
	{
		if ((*it)->getActionNameHash() == name_hash)
			return *it;
	}

	return NULL;
}

IIwGameXomlAction* CIwGameXoml::findAction(const char* name)
{
	return findAction(CIwGameString::CalculateHash(name));
}

void CIwGameXoml::Init()
{
	// Add some basic classes to the XOML tags namespace
	addClass(new CIwGameTemplateFromCreator());		// CIwGameTemplateFrom object 
	addClass(new CIwGameActionsCreator());			// CIwGameActions object 
	addClass(new CIwGameAnimCreator());				// CIwGameAnim object 
	addClass(new CIwGameAnimTimelineCreator());		// CIwGameAnimTimeline object 
	addClass(new CIwGameActorImageCreator());		// CIwGameActorImage object
	addClass(new CIwGameActorTextCreator());		// CIwGameActorText object
	addClass(new CIwGameModifierManagerCreator());	// CIwGameModifierManager object
	addClass(new CIwGameXomlBindingsCreator());		// CIwGameXomlBindings object 
	addClass(new CIwGameStyleCreator());			// CIwGameStyle object 
	addClass(new CIwGameTemplateCreator());			// CIwGameTemplate object 
#if defined(IW_GAME_BOX2D)
	addClass(new CIwGameBox2dMaterialCreator());	// CIwGameBox2dMaterial object 
	addClass(new CIwGameBox2dJointsCreator());		// CIwGameBox2dJoints object 
#endif // IW_GAME_BOX2D_H_
	addClass(new CIwGameSceneCreator());			// CIwGameScene object 
	addClass(new CIwGameCameraCreator());			// CIwGameCamera object 
	addClass(new CIwGameBrushCreator());			// CIwGameImage object 
	addClass(new CIwGameShapeCreator());			// CIwGameShape object 
	addClass(new CIwGameImageCreator());			// CIwGameImage object 
	addClass(new CIwGameResourceGroupCreator());	// CIwGameResourceGroup object 
	addClass(new CIwGameXomlAddVariableCreator());	// CIwGameXomlAddVariable object 
	addClass(new CIwGameXomlLoadCreator());			// CIwGameXomlLoad object 
	addClass(new CIwGameFontCreator());				// CIwGameFont object 
	addClass(new CIwGameCreator());					// CIwGame object

	// Add scene actions
	for (int t = 0; t < CIwGameXomlAction_Scenes::Action_Max; t++)
		addAction(new CIwGameXomlAction_Scenes((CIwGameXomlAction_Scenes::ActionType)t));
	// Add actor actions
	for (int t = 0; t < CIwGameXomlAction_Actors::Action_Max; t++)
		addAction(new CIwGameXomlAction_Actors((CIwGameXomlAction_Actors::ActionType)t));
	// Add global actions
	for (int t = 0; t < CIwGameXomlAction_Global::Action_Max; t++)
		addAction(new CIwGameXomlAction_Global((CIwGameXomlAction_Global::ActionType)t));

}

void CIwGameXoml::InitExtensions()
{
	// Add extensions to XOML tags namescape
	addClass(new CIwGameActorParticlesCreator());	// CIwGameActorParticles object
	addClass(new CIwGameActorConnectorCreator());	// CIwGameActorImage object

	// Add modifiers to modifiers list
	IW_GAME_MODS->addModifier(new CIwGameModFollowHeadingCreator());
}

void CIwGameXoml::Release()
{
	// Clean up actions
	for (_ActionsIterator it = actions_begin(); it != actions_end(); ++it)
	{
		delete *it;
	}
	Actions.clear();

	// Clean up class creators
	for (_Iterator it = begin(); it != end(); ++it)
	{
		delete *it;
	}
	ClassCreators.clear();
}

bool CIwGameXoml::Process(IIwGameXomlResource* parent, const char* filename, bool reset_xml_pools)
{
	if (reset_xml_pools)
		CIwGameXmlParser::PoolsReset();
	CIwGameXmlParser*	xml = new CIwGameXmlParser();

	// Load and parse the xml file
	if (xml->Parse(filename) == eXMLParserError_None)
	{
		// Process the XOML data
		bool error = Process(parent, xml->GetRoot()->GetFirstNode());
		delete xml;
		return error;
	}

	delete xml;
	return false;
}

bool CIwGameXoml::Process(IIwGameXomlResource* parent, CIwGameXmlNode* node, CIwGameString* prefix)
{
	node->ToLower(true);

	for (CIwGameXmlNode::_Iterator it = node->begin(); it != node->end(); it++)
	{
		unsigned int name_hash = (*it)->GetName().getHash();

		// Check to see if we support the class
		for (_Iterator it2 = begin(); it2 != end(); ++it2)
		{
			if ((*it2)->getClassNameHash() == name_hash)
			{
				// We do support the class so call the classes creator to instantiate it
				IIwGameXomlResource* instance = (*it2)->CreateInstance(parent);
				if (instance != NULL)
				{
					// Load up from XOML
					if (!instance->LoadFromXoml(parent, true, *it))
					{
						delete instance;
						CIwGameError::LogError("Error: Near XOML tag ", (*it)->GetName().c_str());
						return false;
					}
				}
				else
				{
					CIwGameError::LogError("Error: No XOML creator present for ", (*it)->GetName().c_str());
					return false;
				}
				break;
			}
		}
	}

	return true;
}

void CIwGameXoml::ApplyStyle(CIwGameStyle* style, CIwGameXmlNode* node)
{
	// Add style properties to nodes
	for (CIwGameStyle::_Iterator it = style->begin(); it != style->end(); ++it)
	{
		node->AddAttribute((*it)->Name, (*it)->Value);
	}
}

// 
// 
//
//
// CIwGameXomlVariable implementation
//
//
//
//
bool CIwGameXomlVariable::checkCondition(eIwGameXomlConditionOperator condition, const CIwGameString& val)
{
	switch (condition)
	{
	case CO_Equal:
		if (Value == val)
			return true;
		break;
	case CO_NotEqual:
		if (!(Value == val))
			return true;
		break;
	case CO_Greater:
		if (Value.GetLength() > val.GetLength())
			return true;
		break;
	case CO_GreaterEqual:
		if (Value.GetLength() >= val.GetLength())
			return true;
		break;
	case CO_Less:
		if (Value.GetLength() < val.GetLength())
			return true;
		break;
	case CO_LessEqual:
		if (Value.GetLength() <= val.GetLength())
			return true;
		break;
	case CO_And:
		Value.FindReset();
		if (Value.Find(val.c_str()) >= 0)
			return true;
		break;
	}

	return false;
}

// 
// 
//
//
// CIwGameXomlVariableBool implementation
//
//
//
//
bool CIwGameXomlVariableBool::checkCondition(eIwGameXomlConditionOperator condition, const CIwGameString& val)
{
	switch (condition)
	{
	case CO_Equal:
	case CO_And:
		if (Value == val)
			return true;
		break;
	case CO_NotEqual:
		if (!(Value == val))
			return true;
		break;
	}

	return false;
}

// 
// 
//
//
// CIwGameXomlVariableFloat implementation
//
//
//
//
bool CIwGameXomlVariableFloat::checkCondition(eIwGameXomlConditionOperator condition, const CIwGameString& val)
{
	float value = val.GetAsFloat();
	switch (condition)
	{
	case CO_Equal:
		if (NativeValue == value)
			return true;
		break;
	case CO_NotEqual:
		if (NativeValue != value)
			return true;
		break;
	case CO_Greater:
		if (NativeValue > value)
			return true;
		break;
	case CO_GreaterEqual:
		if (NativeValue >= value)
			return true;
		break;
	case CO_Less:
		if (NativeValue < value)
			return true;
		break;
	case CO_LessEqual:
		if (NativeValue <= value)
			return true;
		break;
	case CO_And:
		break;
	}

	return false;
}

// 
// 
//
//
// CIwGameXomlVariableInt implementation
//
//
//
//
bool CIwGameXomlVariableInt::checkCondition(eIwGameXomlConditionOperator condition, const CIwGameString& val)
{
	int value = val.GetAsInt();
	switch (condition)
	{
	case CO_Equal:
		if (NativeValue == value)
			return true;
		break;
	case CO_NotEqual:
		if (NativeValue != value)
			return true;
		break;
	case CO_Greater:
		if (NativeValue > value)
			return true;
		break;
	case CO_GreaterEqual:
		if (NativeValue >= value)
			return true;
		break;
	case CO_Less:
		if (NativeValue < value)
			return true;
		break;
	case CO_LessEqual:
		if (NativeValue <= value)
			return true;
		break;
	case CO_And:
		if ((NativeValue & value) == value)
			return true;
		break;
	}

	return false;
}

// 
// 
//
//
// CIwGameXomlVariableVec2 implementation
//
//
//
//
bool CIwGameXomlVariableVec2::checkCondition(eIwGameXomlConditionOperator condition, const CIwGameString& val)
{
	float values[8];
	int count = ((CIwGameString&)val).GetAsListOfFloat(values);
	if (count != 2)
		return false;
	CIwFVec2 value = CIwFVec2(values[0], values[1]);
	switch (condition)
	{
	case CO_Equal:
		if (NativeValue == value)
			return true;
		break;
	case CO_NotEqual:
		if (NativeValue != value)
			return true;
		break;
	case CO_Greater:
		if (NativeValue.GetLengthSquared() > value.GetLengthSquared())
			return true;
		break;
	case CO_GreaterEqual:
		if (NativeValue.GetLengthSquared() >= value.GetLengthSquared())
			return true;
		break;
	case CO_Less:
		if (NativeValue.GetLengthSquared() < value.GetLengthSquared())
			return true;
		break;
	case CO_LessEqual:
		if (NativeValue.GetLengthSquared() <= value.GetLengthSquared())
			return true;
		break;
	}

	return false;
}

// 
// 
//
//
// CIwGameXomlVariableVec3 implementation
//
//
//
//
bool CIwGameXomlVariableVec3::checkCondition(eIwGameXomlConditionOperator condition, const CIwGameString& val)
{
	float values[8];
	int count = ((CIwGameString&)val).GetAsListOfFloat(values);
	if (count != 3)
		return false;
	CIwFVec3 value = CIwFVec3(values[0], values[1], values[2]);
	switch (condition)
	{
	case CO_Equal:
		if (NativeValue == value)
			return true;
		break;
	case CO_NotEqual:
		if (NativeValue != value)
			return true;
		break;
	case CO_Greater:
		if (NativeValue.GetLengthSquared() > value.GetLengthSquared())
			return true;
		break;
	case CO_GreaterEqual:
		if (NativeValue.GetLengthSquared() >= value.GetLengthSquared())
			return true;
		break;
	case CO_Less:
		if (NativeValue.GetLengthSquared() < value.GetLengthSquared())
			return true;
		break;
	case CO_LessEqual:
		if (NativeValue.GetLengthSquared() <= value.GetLengthSquared())
			return true;
		break;
	}

	return false;
}

// 
// 
//
//
// CIwGameXomlVariableVec4 implementation
//
//
//
//
bool CIwGameXomlVariableVec4::checkCondition(eIwGameXomlConditionOperator condition, const CIwGameString& val)
{
	float values[8];
	int count = ((CIwGameString&)val).GetAsListOfFloat(values);
	if (count != 4)
		return false;
	CIwFVec4 value = CIwFVec4(values[0], values[1], values[2], values[3]);
	switch (condition)
	{
	case CO_Equal:
		if (NativeValue == value)
			return true;
		break;
	case CO_NotEqual:
		if (NativeValue != value)
			return true;
		break;
	case CO_Greater:
		if (NativeValue.GetLengthSquared() > value.GetLengthSquared())
			return true;
		break;
	case CO_GreaterEqual:
		if (NativeValue.GetLengthSquared() >= value.GetLengthSquared())
			return true;
		break;
	case CO_Less:
		if (NativeValue.GetLengthSquared() < value.GetLengthSquared())
			return true;
		break;
	case CO_LessEqual:
		if (NativeValue.GetLengthSquared() <= value.GetLengthSquared())
			return true;
		break;
	}

	return false;
}

// 
// 
//
//
// CIwGameXomlVariableCondition implementation
//
//
//
//
void CIwGameXomlVariableCondition::setValue(const char* value)
{
	Value = value;

	// Build condition list
	// Parse out string parameters
	CIwList<CIwSVec2> params;
	int pos, len;
	while (1)
	{
		pos = Value.getFindIndex();
		len = Value.GetNextString();
		if (len <= 0) break;
		Value.StepFindIndex(1);
		params.push_back(CIwSVec2(pos, len));
	}

	// Go through parameters to find out what they are
	CIwGameString variable;
	CIwGameString op;
	CIwGameString val;
	CIwGameString join_op;
	for (CIwList<CIwSVec2>::iterator it = params.begin(); it != params.end(); it++)
	{
		// Get variable name
		variable = Value.getSubString((*it).x, (*it).y); it++;
		if (it == params.end())
		{
			CIwGameError::LogError("Warning: Variable found but no operator or value");
			return;
		}

		// Get operator
		op = Value.getSubString((*it).x, (*it).y); it++;
		if (it == params.end())
		{
			CIwGameError::LogError("Warning: operator found but no value");
			return;
		}

		// Determine operator
		eIwGameXomlConditionOperator type;
		op.ToLower();
		if (op == "==")	type = CO_Equal; else
		if (op == "!=")	type = CO_NotEqual; else
		if (op == "gt")	type = CO_Greater; else
		if (op == "lt")	type = CO_Less; else
		if (op == "gte") type = CO_GreaterEqual; else
		if (op == "lte") type = CO_LessEqual; else
		if (op == "and") type = CO_And; else
		{
			CIwGameError::LogError("Warning: operator invalid - ", op.c_str());
			return;
		}

		// Get value
		val = Value.getSubString((*it).x, (*it).y); it++;

		// Creat the condition
		CIwGameXomlCondition* condition = new CIwGameXomlCondition();
		condition->Value = val;
		condition->Operator = type;
		condition->VariabledName = variable.getHash();
#if defined(_DEBUG)
		condition->_VariabledName = variable;
#endif

		// Add condition to the conditions list
		Conditions.push_back(condition);

		// If no more data left then exit
		if (it == params.end())
			break;

		// Get join operator if its present
		join_op = Value.getSubString((*it).x, (*it).y);
		join_op.ToLower();
		if (join_op == "and")
			condition->Join = CJO_And;
		else
		if (join_op == "or")
			condition->Join = CJO_Or;
		else
		{
			CIwGameError::LogError("Warning: joining operator invalid", join_op.c_str());
			break;
		}

		if (it == params.end())
		{
			CIwGameError::LogError("Warning: joining operator is present but no additional conditions available");
			break;
		}
	}
}

bool CIwGameXomlVariableCondition::isTrue()
{
	bool final_result = false;

	// Check the condition variables
	CIwGameXomlCondition* prev = NULL;
	for (CIwList<CIwGameXomlCondition*>::iterator it = Conditions.begin(); it != Conditions.end(); ++it)
	{
		// Find the variable
		CIwGameXomlVariable* var = Parent->findVariable((*it)->VariabledName);
		if (var == NULL)
		{
			// If parent is not the global resource manager then check to see if the variable is present in the global resuorce manager
			if (Parent != IW_GAME_GLOBAL_RESOURCES->getVariableManager())
			{
				var = IW_GAME_GLOBAL_RESOURCES->getVariableManager()->findVariable((*it)->VariabledName);
			}
		}
		if (var != NULL)
		{
			bool result = var->checkCondition((*it)->Operator, (*it)->Value);

			// If joined by AND operator then rest of conditions are irrelevant
			if (!result && (*it)->Join == CJO_And)
			{
				NativeValue = false;
				return false;
			}

			if (prev == NULL)
				final_result = result;
			else
			{
				if (prev->Join == CJO_And)
					final_result = final_result && result;
				else
				if (prev->Join == CJO_Or)
					final_result = final_result || result;
			}
		}
		else
		{
#if defined(_DEBUG)
			CIwGameError::LogError("Warning: variable could not be fuond - ", var->Name.c_str());
#endif	// _DEBUG
		}
		prev = *it;
	}
	NativeValue = final_result;

	return NativeValue;
}

// 
// 
//
//
// CIwGameXomlVariableManager implementation
//
//
//
//

void CIwGameXomlVariableManager::addVariable(CIwGameXomlVariable* variable)
{
#if defined(_DEBUG)
	if (findVariable(variable->Name.getHash()))
	{
		CIwGameError::LogError("Warning: Variable already exists, duplicate was ignored - ", variable->Name.c_str());
		delete variable;
		return;
	}
#endif	//(_DEBUG)
	Variables.push_back(variable);
	variable->Parent = this;
}

void CIwGameXomlVariableManager::addVariable(const char* name, eCIwGameXomlVariableType type)
{
#if defined(_DEBUG)
	if (findVariable(CIwGameString::CalculateHash(name)))
	{
		CIwGameError::LogError("Warning: Variable already exists, duplicate was ignored - ", name);
		return;
	}
#endif	//(_DEBUG)
	Variables.push_back(CreateVariable(name, type));
}

void CIwGameXomlVariableManager::addVariable(const char* name, eCIwGameXomlVariableType type, const char* value)
{
#if defined(_DEBUG)
	if (findVariable(CIwGameString::CalculateHash(name)))
	{
		CIwGameError::LogError("Warning: Variable already exists, duplicate was ignored - ", name);
		return;
	}
#endif	//(_DEBUG)
	CIwGameXomlVariable* v = CreateVariable(name, type);
	v->setValue(value);
	Variables.push_back(v);
}

void CIwGameXomlVariableManager::setVariable(unsigned int name_hash, const char* value)
{
	CIwGameXomlVariable* var = findVariable(name_hash);
	if (var != NULL)
	{
		var->setValue(value);
	}
}

void CIwGameXomlVariableManager::setVariable(const char* name, const char* value)
{
	setVariable(CIwGameString::CalculateHash(name), value);
}

CIwGameXomlVariable* CIwGameXomlVariableManager::findVariable(unsigned int name_hash)
{
	for (_Iterator it = Variables.begin(); it != Variables.end(); ++it)
	{
		if ((*it)->Name.getHash() == name_hash)
		{
			return *it;
		}
	}

	return NULL;
}

CIwGameXomlVariable* CIwGameXomlVariableManager::findVariable(const char* name)
{
	return findVariable(CIwGameString::CalculateHash(name));
}

void CIwGameXomlVariableManager::clearVariables()
{
	// Remove and delete all images from the manager
	for (_Iterator it = Variables.begin(); it != Variables.end(); ++it)
	{
		delete *it;
	}
	Variables.clear();
}

CIwGameXomlVariable* CIwGameXomlVariableManager::CreateVariable(const char* name, eCIwGameXomlVariableType type)
{
	// Create variable of specified type
	switch (type)
	{
		case VT_String: 
			{
				CIwGameXomlVariable* var = new CIwGameXomlVariable();
				var->Parent = this;
				var->Name = name;
				var->Type = type;

				return var;
			}
			break;
		case VT_Bool: 
			{
				CIwGameXomlVariableBool* var = new CIwGameXomlVariableBool();
				var->Parent = this;
				var->Name = name;
				var->Type = type;

				return var;
			}
			break;
		case VT_Float: 
			{
				CIwGameXomlVariableFloat* var = new CIwGameXomlVariableFloat();
				var->Parent = this;
				var->Name = name;
				var->Type = type;

				return var;
			}
			break;
		case VT_Int: 
			{
				CIwGameXomlVariableInt* var = new CIwGameXomlVariableInt();
				var->Parent = this;
				var->Name = name;
				var->Type = type;

				return var;
			}
			break;
		case VT_Vec2: 
			{
				CIwGameXomlVariableVec2* var = new CIwGameXomlVariableVec2();
				var->Parent = this;
				var->Name = name;
				var->Type = type;

				return var;
			}
			break;
		case VT_Vec3: 
			{
				CIwGameXomlVariableVec3* var = new CIwGameXomlVariableVec3();
				var->Parent = this;
				var->Name = name;
				var->Type = type;

				return var;
			}
			break;
		case VT_Vec4: 
			{
				CIwGameXomlVariableVec4* var = new CIwGameXomlVariableVec4();
				var->Parent = this;
				var->Name = name;
				var->Type = type;

				return var;
			}
			break;
		case VT_Condition: 
			{
				CIwGameXomlVariableCondition* var = new CIwGameXomlVariableCondition();
				var->Parent = this;
				var->Name = name;
				var->Type = type;

				return var;
			}
			break;
	}

	return NULL;
}

CIwGameXomlVariable* CIwGameXomlVariableManager::FindVariable(const char* name, IIwGameXomlResource* container)
{
	return FindVariable(CIwGameString::CalculateHash(name), container);
}

CIwGameXomlVariable* CIwGameXomlVariableManager::FindVariable(unsigned int name_hash, IIwGameXomlResource* container)
{
	CIwGameXomlVariable* var = NULL;
	
	// If container class is an actor then check the actors scenes variables manager
	if (container != NULL && container->getClassTypeHash() == CIwGameXomlNames::Actor_Hash)
		container = ((CIwGameActor*)container)->getScene();
	// If container class is a scene then check the scenes variables manager
	if (container != NULL && container->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
		var = (CIwGameXomlVariable*)((CIwGameScene*)container)->getVariableManager()->findVariable((name_hash));

	// If the variable has no container or the variable was not found in the supplied container then check the global variables manager
	if (var == NULL)
		var = (CIwGameXomlVariable*)IW_GAME_GLOBAL_RESOURCES->getVariableManager()->findVariable((name_hash));

	return var;
}

// 
// 
//
//
// CIwGameXomlBindings implementation
//
//
//
//

void CIwGameXomlBindings::addBinding(CIwGameXomlBinding* binding)
{
	Bindings.push_back(binding);
}

void CIwGameXomlBindings::addBinding(const CIwGameString& property_name, CIwGameXomlVariable* variable)
{
	CIwGameXomlBinding* binding = new CIwGameXomlBinding();
	binding->Property = property_name.getHash();
#if defined(_DEBUG)
	binding->_Property = property_name;
#endif	// _DEBUG
	binding->Variable = variable;
	Bindings.push_back(binding);
}


void CIwGameXomlBindings::clearBindings()
{
	// Remove and delete all images from the manager
	for (_Iterator it = Bindings.begin(); it != Bindings.end(); ++it)
	{
		delete *it;
	}
	Bindings.clear();
}

bool CIwGameXomlBindings::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	// Process bindings attributes
	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Name_Hash)
		{
			setName((*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::Tag_Hash)
			setTag((*it)->GetValue().c_str());
	}

	// Prrocess bindings
	for (CIwGameXmlNode::_Iterator it2 = node->begin(); it2 != node->end(); ++it2)
	{
		unsigned int name_hash = (*it2)->GetName().getHash();
		if (name_hash == CIwGameXomlNames::Binding_Hash)
		{
			CIwGameXomlBinding* binding = new CIwGameXomlBinding();

			// Get binding data
			for (CIwGameXmlNode::_AttribIterator it = (*it2)->attribs_begin(); it != (*it2)->attribs_end(); ++it)
			{
				unsigned int attrib_hash = (*it)->getName().getHash();

				if (attrib_hash == CIwGameXomlNames::Property_Hash)
				{
					(*it)->GetValue().ToLower();
					binding->Property = (*it)->GetValue().getHash();
#if defined(_DEBUG)
					binding->_Property = (*it)->GetValue();
#endif	//_DEBUG
				}
				else
				if (attrib_hash == CIwGameXomlNames::Variable_Hash)
				{
					CIwGameXomlVariable* var = CIwGameXomlVariableManager::FindVariable((*it)->GetValue().getHash(), parent);
					if (var == NULL)
					{
						CIwGameError::LogError("Error: binding variable not found - ", (*it)->GetValue().c_str());
					}
					binding->Variable = var;
				}
			}
			// Save action
			Bindings.push_back(binding);
		}
	}

	// If we are declared inside a scene then bindings are local to the scene
	if (parent != NULL && parent->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
	{
		((CIwGameScene*)parent)->getResourceManager()->addResource(this);
	}
	else
	{
		IW_GAME_GLOBAL_RESOURCES->getResourceManager()->addResource(this);
	}


	return true;
}

//
// 
// 
//
// CIwGameXomlAddVariable Implementation
// 
// 
// 
//
bool CIwGameXomlAddVariable::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	CIwGameString name, value, type;

	// Process variable specific attributes
	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Name_Hash)
			name = (*it)->GetValue();
		else
		if (name_hash == CIwGameXomlNames::Tag_Hash)
			setTag((*it)->GetValue().c_str());
		else
		if (name_hash == CIwGameXomlNames::Value_Hash)
			value = (*it)->GetValue();
		else
		if (name_hash == CIwGameXomlNames::Type_Hash)
			type = (*it)->GetValue();
	}

	if (!name.IsEmpty())
	{
		// Determine type of variable
		eCIwGameXomlVariableType vtype = VT_String;
		if (!type.IsEmpty())
		{
			unsigned int type_hash = type.getHash();
			if (type_hash == CIwGameXomlNames::bool_Hash)
				vtype = VT_Bool;
			else
			if (type_hash == CIwGameXomlNames::float_Hash)
				vtype = VT_Float;
			else
			if (type_hash == CIwGameXomlNames::int_Hash)
				vtype = VT_Int;
			else
			if (type_hash == CIwGameXomlNames::vec2_Hash)
				vtype = VT_Vec2;
			else
			if (type_hash == CIwGameXomlNames::vec3_Hash)
				vtype = VT_Vec3;
			else
			if (type_hash == CIwGameXomlNames::vec4_Hash)
				vtype = VT_Vec4;
			else
			if (type_hash == CIwGameXomlNames::condition_Hash)
				vtype = VT_Condition;
		}

		// If we are declared inside a scene then variable is local to the scene else global
		if (parent != NULL && parent->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
		{
			((CIwGameScene*)parent)->getVariableManager()->addVariable(name.c_str(), vtype, value.c_str());
		}
		else
		{
			IW_GAME_GLOBAL_RESOURCES->getVariableManager()->addVariable(name.c_str(), vtype, value.c_str());
		}
	}
	else
	{
		CIwGameError::LogError("Error: XOML - Variable must be named");
		delete this;
		return false;
	}

	delete this;

	return true;
}

//
// 
// 
//
// CIwGameXomlLoad Implementation
// 
// 
// 
//
bool CIwGameXomlLoad::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	// Process LoadXoml specific attributes
	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::File_Hash)
		{
			IW_GAME_XOML->Process(parent, (*it)->GetValue().c_str(), false);
		}
	}
	delete this;

	return true;
}

